package fms

import (
	"fmt"
	"github.com/shopspring/decimal"
	"hongze/fms_api/models/fms"
	"hongze/fms_api/services/alarm_msg"
	"hongze/fms_api/utils"
	"time"
)

// CalculateContractPaymentType 计算到款登记付款方式
// 计算公式: 到款金额 / ( 合同金额 / round( (合同结束日期 - 合同开始日期) / 365) )
func CalculateContractPaymentType(paymentAmount, contractAmount, dayDiff float64) (payEnum int) {
	payEnum = fms.ContractPaymentPayTypeAbnormal // 默认异常
	if paymentAmount <= 0 || contractAmount <= 0 || dayDiff <= 0 {
		return
	}

	// 年份四舍五入
	days := decimal.NewFromFloat(dayDiff)
	yearDays := decimal.NewFromFloat(365)
	yearRound := days.DivRound(yearDays, 2)
	if yearRound.IsZero() {
		return
	}
	years := yearRound.Add(decimal.NewFromFloat(0.5)).Floor()
	if years.IsZero() {
		return
	}
	// 分母
	contractDec := decimal.NewFromFloat(contractAmount)
	contractDeno := contractDec.DivRound(years, 2)
	if contractDeno.IsZero() {
		return
	}
	paymentDec := decimal.NewFromFloat(paymentAmount)
	// 结果
	resultDec := paymentDec.DivRound(contractDeno, 2)

	// 标准比例
	yearPay := decimal.NewFromFloat(1)
	halfYearPay := decimal.NewFromFloat(0.5)
	quarterPay := decimal.NewFromFloat(0.25)
	zeroPay := decimal.NewFromFloat(0)

	payEnum = fms.ContractPaymentPayTypeOther
	// 异常
	if yearPay.LessThan(resultDec) || resultDec.LessThanOrEqual(zeroPay) {
		payEnum = fms.ContractPaymentPayTypeAbnormal
		return
	}
	// 年付
	if resultDec.Equal(yearPay) {
		payEnum = fms.ContractPaymentPayTypeYear
		return
	}
	// 半年付
	if resultDec.Equal(halfYearPay) {
		payEnum = fms.ContractPaymentPayTypeHalfYear
		return
	}
	// 季付
	if resultDec.Equal(quarterPay) {
		payEnum = fms.ContractPaymentPayTypeQuarter
		return
	}
	return
}

// MergeInvoiceList2InvoicePaymentCensusInfo 合同登记-合并开票到款列表
func MergeInvoiceList2InvoicePaymentCensusInfo(invoiceList []*fms.ContractInvoice, paymentList []*fms.ContractInvoice, serviceList []*fms.ContractService, serviceAmountList []*fms.ContractPaymentServiceAmount) (mergeList []*fms.InvoicePaymentCensusInfo) {
	invoiceLen := len(invoiceList)
	paymentLen := len(paymentList)
	if invoiceLen == 0 && paymentLen == 0 {
		return
	}

	// 到款套餐分配信息
	amountMap := make(map[string]*fms.ContractPaymentServiceAmount)
	for i := range serviceAmountList {
		k := fmt.Sprintf("%d-%d", serviceAmountList[i].ContractPaymentId, serviceAmountList[i].ServiceTemplateId)
		amountMap[k] = serviceAmountList[i]
	}
	serviceAmountMap := make(map[int][]*fms.ContractPaymentServiceAmountItem)
	for i := range paymentList {
		l := make([]*fms.ContractPaymentServiceAmountItem, 0)
		for ii := range serviceList {
			v := new(fms.ContractPaymentServiceAmountItem)
			v.ServiceTemplateId = serviceList[ii].ServiceTemplateId
			v.ServiceTemplateName = serviceList[ii].Title
			k := fmt.Sprintf("%d-%d", paymentList[i].ContractInvoiceId, serviceList[ii].ServiceTemplateId)
			a := amountMap[k]
			if a != nil {
				v.ContractPaymentServiceAmountId = a.ContractPaymentServiceAmountId
				v.ContractPaymentId = a.ContractPaymentId
				v.Amount = a.Amount
			}
			l = append(l, v)
		}
		serviceAmountMap[paymentList[i].ContractInvoiceId] = l
	}

	// 开票和到款列表, 条数多的为最后的合并条数
	mergeList = make([]*fms.InvoicePaymentCensusInfo, 0)
	if invoiceLen >= paymentLen {
		for i := range invoiceList {
			v := new(fms.InvoicePaymentCensusInfo)
			v.InvoiceId = invoiceList[i].ContractInvoiceId
			v.InvoiceDate = invoiceList[i].InvoiceDate.Format(utils.FormatDate)
			v.InvoiceAmount = invoiceList[i].OriginAmount
			v.SellerId = invoiceList[i].SellerId
			v.SellerName = invoiceList[i].SellerName
			v.SellerGroupId = invoiceList[i].SellerGroupId
			v.SellerGroupName = invoiceList[i].SellerGroupName
			// 直接取对应键的到款列表
			if i+1 <= paymentLen {
				payItem := paymentList[i]
				if payItem != nil {
					v.PaymentId = payItem.ContractInvoiceId
					v.PaymentDate = payItem.InvoiceDate.Format(utils.FormatDate)
					v.PaymentAmount = payItem.OriginAmount
					v.PayType = payItem.PayType
					v.ServiceAmountList = serviceAmountMap[payItem.ContractInvoiceId]
				}
			}
			mergeList = append(mergeList, v)
		}
		return
	}

	// 到款多于开票
	if paymentLen > invoiceLen {
		for i := range paymentList {
			v := new(fms.InvoicePaymentCensusInfo)
			v.PaymentId = paymentList[i].ContractInvoiceId
			v.PaymentDate = paymentList[i].InvoiceDate.Format(utils.FormatDate)
			v.PaymentAmount = paymentList[i].OriginAmount
			v.PayType = paymentList[i].PayType
			v.ServiceAmountList = serviceAmountMap[paymentList[i].ContractInvoiceId]
			// 直接取对应键的开票
			if i+1 <= invoiceLen {
				invoiceItem := invoiceList[i]
				if invoiceItem != nil {
					v.InvoiceId = invoiceItem.ContractInvoiceId
					v.InvoiceDate = invoiceItem.InvoiceDate.Format(utils.FormatDate)
					v.InvoiceAmount = invoiceItem.OriginAmount
					v.SellerId = invoiceItem.SellerId
					v.SellerName = invoiceItem.SellerName
					v.SellerGroupId = invoiceList[i].SellerGroupId
					v.SellerGroupName = invoiceList[i].SellerGroupName
				}
			}
			mergeList = append(mergeList, v)
		}
		return
	}
	return
}

// SummaryInvoicePaymentByContractRegisterId 汇总合同登记的开票到款, 即一一进行对应存入汇总表
func SummaryInvoicePaymentByContractRegisterId(registerId int) {
	var err error
	defer func() {
		if err != nil {
			alarm_msg.SendAlarmMsg(fmt.Sprintf("汇总开票到款失败, ErrMsg: \n%s", err.Error()), 3)
		}
	}()

	// 获取开票到款信息
	cond := `contract_register_id = ?`
	pars := make([]interface{}, 0)
	pars = append(pars, registerId)
	list, e := fms.GetContractInvoiceItemList(cond, pars)
	if e != nil {
		err = fmt.Errorf("获取开票到款列表失败, Err: %s", e.Error())
		return
	}
	invoiceIds := make([]int, 0)
	paymentIds := make([]int, 0)
	for i := range list {
		if list[i].InvoiceType == fms.ContractInvoiceTypeMake {
			invoiceIds = append(invoiceIds, list[i].ContractInvoiceId)
			continue
		}
		if list[i].InvoiceType == fms.ContractInvoiceTypePay {
			paymentIds = append(paymentIds, list[i].ContractInvoiceId)
		}
	}
	invoiceLen := len(invoiceIds)
	paymentLen := len(paymentIds)

	// 汇总数据
	nowTime := time.Now().Local()
	summaryList := make([]*fms.InvoicePaymentSummary, 0)
	if invoiceLen >= paymentLen {
		for i := range invoiceIds {
			v := new(fms.InvoicePaymentSummary)
			v.RegisterId = registerId
			v.InvoiceId = invoiceIds[i]
			v.CreateTime = nowTime
			v.ModifyTime = nowTime
			// 取对应key的到款ID
			if i+1 <= paymentLen {
				v.PaymentId = paymentIds[i]
			}
			summaryList = append(summaryList, v)
		}
	}
	if paymentLen > invoiceLen {
		for i := range paymentIds {
			v := new(fms.InvoicePaymentSummary)
			v.RegisterId = registerId
			v.PaymentId = paymentIds[i]
			v.CreateTime = nowTime
			v.ModifyTime = nowTime
			// 取对应key的开票ID
			if i+1 <= invoiceLen {
				v.InvoiceId = invoiceIds[i]
			}
			summaryList = append(summaryList, v)
		}
	}

	// 删除并新增汇总数据
	summaryOB := new(fms.InvoicePaymentSummary)
	if e = summaryOB.DeleteAndCreate(registerId, summaryList); e != nil {
		err = fmt.Errorf("新增汇总数据失败, Err: %s", e.Error())
	}
	return
}
